Package: silicate
Title: Common Forms for Complex Hierarchical and Relational Data
        Structures
Version: 0.7.1
Authors@R: c(person("Michael D.","Sumner", role = c("aut", "cre"), email = "mdsumner@gmail.com"), 
             person("John", "Corbett", role = "ctb", comment = "the original inspiration"), 
             person("Simon", "Wotherspoon", role = "ctb"), 
             person("Kent", "Johnson", role = "dtc"), 
             person("Mark", "Padgham", role = "aut")
             )
Description: Generate common data forms for complex data suitable for conversions and
 transmission by decomposition as paths or primitives. Paths are sequentially-linked records, 
 primitives are basic atomic elements and both can model many forms and be grouped into hierarchical 
 structures.  The universal models 'SC0' (structural) and 'SC' (labelled, relational) are composed of 
 edges and can represent any hierarchical form. Specialist models 'PATH', 'ARC' and 'TRI' provide the 
 most common intermediate forms used for converting from one form to another. The methods are
 inspired by the simplicial complex <https://en.wikipedia.org/wiki/Simplicial_complex> and 
 provide intermediate forms that relate spatial data structures to this mathematical construct. 
Depends: R (>= 3.4.0)
License: GPL-3
Encoding: UTF-8
LazyData: true
Suggests: covr, knitr, rmarkdown, sp, testthat (>= 2.1.0), trip, vdiffr
Imports: dplyr, gibble (>= 0.4.0), methods, purrr, rlang, decido,
        tibble, unjoin (>= 0.1.0), grDevices, graphics, stats,
        magrittr, gridBase, crsmeta (>= 0.3.0)
VignetteBuilder: knitr
URL: https://github.com/hypertidy/silicate
BugReports: https://github.com/hypertidy/silicate/issues
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2023-01-03 10:57:19 UTC; mdsumner
Author: Michael D. Sumner [aut, cre],
  John Corbett [ctb] (the original inspiration),
  Simon Wotherspoon [ctb],
  Kent Johnson [dtc],
  Mark Padgham [aut]
Maintainer: Michael D. Sumner <mdsumner@gmail.com>
Repository: CRAN
Date/Publication: 2023-01-06 18:50:28 UTC
Built: R 4.6.0; ; 2026-04-30 03:38:14 UTC; unix
