#from https://github.com/r-spatial/mapview/issues/99#issuecomment-328711275

#> Attached is a zipped shapefile containing 144 flight tracks of aircraft #departing runway
#> 33L at Boston Logan airport on June 28, 2017. Data is from an ADS-B recorder so it is 
#> kind of rough. Each point includes lat, lon, altitude in feet and time in EDT. #Unzip and 
#> read > with
#> st_read('tracks.shp')
#>

#saved as inst/exdata/flight_tracks/tracks.shp/prj/dbf/shx

